import argparse
import datetime
import json
import os
import socket
import subprocess
import sys
import tempfile


try:
    from pip._internal.operations import freeze
except ImportError:  # pip < 10.0
    from pip.operations import freeze

from eos import git


def make_fancy_output_dir(dirname=None,
                          args=None,
                          time_format='%Y-%m-%d-%H-%M-%S-%f',
                          dir_suffix_name='',
                          save_environ=True,
                          save_command=True,
                          save_git=True,
                          save_gitignore=True,
                          save_pip=True):
    """Create fancy output directory.

    Parameters
    ----------
    dirname : str or None
        if this is None, create tmporary directory.
    args : argparse.Namespace or None
        if args is specified, dump args to created directory.
    time_format : str or None
        time format. If time_format is None,
        time_format directory will not be created.
    dir_suffix_name : str
        suffix name of directory
    save_environ : bool
        if True, dump environemnt values to environ.txt.
    save_command : bool
        if True, dump input command value to command.txt.
    save_git : bool
        if True and under git control, save git information.
    save_gitignore : bool
        if True, add gitignore to output directory.
    save_pip : bool
        if True, save 'pip freeze' result to output directory.

    Returns
    -------
    outdir : str
        path of created directory.

    Examples
    --------
    >>> from eos import make_fancy_output_dir
    >>> make_fancy_output_dir('/tmp/results')
    '/tmp/results/iory-mac-2020-03-27-18-52-49-215104-67099'
    >>> make_fancy_output_dir(time_format=None)
   '/tmp/vm7bwis1'
    """
    if time_format is not None:
        time_str = datetime.datetime.now().strftime(time_format)
        time_str = '-'.join([socket.gethostname(), time_str, str(os.getpid())])
        time_str += dir_suffix_name
    else:
        time_str = ''
    if dirname is not None:
        # create directory
        if os.path.exists(dirname):
            if not os.path.isdir(dirname):
                raise RuntimeError(
                    '{} is not a directory'.format(dirname))
        outdir = os.path.join(dirname, time_str)
        if os.path.exists(outdir):
            raise RuntimeError('{} exists'.format(outdir))
        else:
            os.makedirs(outdir)
    else:
        outdir = tempfile.mkdtemp(prefix=time_str)

    if args is not None:
        # Save all the arguments
        with open(os.path.join(outdir, 'args.txt'), 'w') as f:
            if isinstance(args, argparse.Namespace):
                args = vars(args)
            f.write(json.dumps(args, ensure_ascii=False, indent=4))
            f.write('\n')

    if save_environ:
        # Save all the environment variables
        with open(os.path.join(outdir, 'environ.txt'), 'w') as f:
            f.write(json.dumps(dict(os.environ),
                               ensure_ascii=False,
                               indent=4))
            f.write('\n')

    if save_command:
        # Save the command
        with open(os.path.join(outdir, 'command.txt'), 'w') as f:
            f.write(' '.join(sys.argv))
            f.write('\n')

    if save_git and git.exists and git.is_under_git_control():
        # Save `git rev-parse HEAD` (SHA of the current commit)
        with open(os.path.join(outdir, 'git-head.txt'), 'wb') as f:
            f.write(subprocess.check_output('git rev-parse HEAD'.split()))

        # Save `git status`
        with open(os.path.join(outdir, 'git-status.txt'), 'wb') as f:
            f.write(subprocess.check_output('git status'.split()))

        # Save `git log`
        with open(os.path.join(outdir, 'git-log.txt'), 'wb') as f:
            f.write(subprocess.check_output('git log'.split()))

        # Save `git diff`
        with open(os.path.join(outdir, 'git-diff.txt'), 'wb') as f:
            f.write(subprocess.check_output('git diff'.split()))

    if save_gitignore:
        with open(os.path.join(outdir, '.gitignore'), 'w') as f:
            f.write('*\n')

    if save_pip:
        with open(os.path.join(outdir, 'pip-freeze.txt'), 'w') as f:
            f.write("\n".join(list(freeze.freeze())))
            f.write('*\n')

    return outdir



# flake8: noqa

import pkg_resources


__version__ = pkg_resources.get_distribution("extendedos").version


from eos.compress import make_tarfile
from eos.make_fancy_output_dir import make_fancy_output_dir
from eos.makedirs import makedirs
from eos.measure import measure
from eos.run_many import run_many



import multiprocessing
import os
import signal
import subprocess
import time


def run_many(cmd, n, jobs=None, sleep_time=1.0, verbose=True):
    """Running commands in parallel.

    Parameters
    ----------
    cmd : str
        command to execute.
    n : int
        number of commands to execute.
    jobs : None or int
        number of parallel execution. If `None`,
        automatically set min(multiprocessing.cpu_count(), n).
    sleep_time : float
        sleep time between commands to be executed.
    verbose : bool
        If `True`, print information.
    """
    if jobs is None:
        jobs = min(n, multiprocessing.cpu_count())

    if verbose:
        start = time.time()

    try:
        procs = []
        num_finish = 0
        for i in range(n):
            procs.append(subprocess.Popen(cmd, shell=True))
            time.sleep(sleep_time)
            while len(procs) >= jobs:
                for p in procs:
                    ret = p.poll()
                    if ret is not None:
                        procs.remove(p)
                        num_finish += 1
                        if verbose:
                            print("finish {} processes".format(num_finish))
                        break
                else:
                    time.sleep(sleep_time)
        while procs:
            for p in procs:
                ret = p.poll()
                if ret is not None:
                    procs.remove(p)
                    num_finish += 1
                    if verbose:
                        print("finish {}processes".format(num_finish))
                    break
            else:
                time.sleep(sleep_time)
    except KeyboardInterrupt:
        for p in procs:
            os.killpg(os.getpgid(p.pid), signal.SIGINT)

    if verbose:
        elapsed_time = time.time() - start
        print("elapsed_time: {0}".format(elapsed_time) + "[sec]")



#!/usr/bin/env python

import argparse
from argparse import RawTextHelpFormatter

from eos import run_many


def main():
    parser = argparse.ArgumentParser(
        formatter_class=RawTextHelpFormatter,
        description='Execute command multiple times.\n\n'
        'Examples\n'
        '--------\n'
        'run-many "python -c \'import os; print(os.getpid())\'" '
        '-n 8 --jobs 2 --sleep-time 0.1 --verbose')
    parser.add_argument('cmd', type=str,
                        help='Input command.')
    parser.add_argument('--jobs', '-j', type=int, default=16,
                        help='Number of parallel process')
    parser.add_argument('--num', '-n', type=int, default=32,
                        help='Number of commands to execute.')
    parser.add_argument('--sleep-time', type=float, default=1.0,
                        help='Sleep interval.')
    parser.add_argument('--verbose', action='store_true',
                        help='Print information')
    args = parser.parse_args()
    run_many(args.cmd, args.num, jobs=args.jobs,
             sleep_time=args.sleep_time, verbose=args.verbose)


if __name__ == '__main__':
    main()



import os

from eos.pycompat import PY2


def makedirs(name, mode=0o777, exist_ok=True):
    """An wrapper of os.makedirs that accepts exist_ok.

    Parameters
    ----------
    name : str
        path of directory
    exist_ok : bool
        if True, accepts the existence of the directory.

    Examples
    --------
    >>> from eos import makedirs
    >>> makedirs('/tmp/result_directory')
    """
    name = str(name)
    if PY2:
        try:
            os.makedirs(name, mode)
        except OSError:
            if not (exist_ok and os.path.isdir(name)):
                raise OSError
    else:
        os.makedirs(name, mode, exist_ok=exist_ok)



import os.path as osp
import tarfile


def make_tarfile(output_filename, source_dir=None):
    """Make tarfile

    Parameters
    ----------
    output_filename : str
        output filename. If source_dir is None, output_filename is considered
        as the source_dir name.
    source_dir : str or None
        target directory to compress.

    Returns
    -------
    output_filename : str
        output filename
    """
    if source_dir is None:
        source_dir = output_filename
        output_filename = "{}.tar.gz".format(source_dir)
    with tarfile.open(output_filename, "w:gz") as tar:
        tar.add(source_dir, arcname=osp.basename(source_dir))
    return output_filename



import contextlib
import datetime as datetime_module
import sys


if sys.version_info[:2] >= (3, 6):
    time_isoformat = datetime_module.time.isoformat
else:
    def time_isoformat(time, timespec='microseconds'):
        assert isinstance(time, datetime_module.time)
        if timespec != 'microseconds':
            raise NotImplementedError
        result = '{:02d}:{:02d}:{:02d}.{:06d}'.format(
            time.hour, time.minute, time.second, time.microsecond
        )
        assert len(result) == 15
        return result


if sys.version_info[:2] >= (3, 7):
    time_fromisoformat = datetime_module.time.fromisoformat
else:
    def time_fromisoformat(isoformat_str):
        hour, minute, second, microsecond = map(
            int,
            isoformat_str.replace('.', ':').split(':'))
        return datetime_module.time(hour, minute, second, microsecond)


def timedelta_isoformat(timedelta, timespec='microseconds'):
    time = (datetime_module.datetime.min + timedelta).time()
    return time_isoformat(time, timespec)


@contextlib.contextmanager
def measure(log_name='Total elapsed time',
            file=sys.stderr):
    """Measure time with contextlib.

    Parameters
    ----------
    log_name : str
        log name
    file : writable object
        writable object such as sys.stderr.

    Examples
    --------
    >>> from eos import measure
    >>> with measure('measure example'):
    ...     time.sleep(2.0)
    measure example: 00:00:02.002104
    """
    start_time = datetime_module.datetime.now()
    yield
    duration = datetime_module.datetime.now() - start_time
    now_string = timedelta_isoformat(duration)
    file.write('{log_name}: {now_string}\n'.format(
        **locals()))



from distutils.spawn import find_executable
import os
import subprocess


_search_path = os.environ['PATH']
_git_executable = find_executable('git', path=_search_path)

exists = _git_executable is not None


def is_under_git_control():
    """Checking under git control

    Returns
    -------
    ret : bool
        result of under git control.
    """
    if not exists:
        return False
    FNULL = open(os.devnull, 'w')
    try:
        subprocess.check_call(['git', 'rev-parse'], stdout=FNULL, stderr=FNULL)
    except subprocess.CalledProcessError:
        return False
    return True



from __future__ import print_function

import shlex
import subprocess
import sys

from setuptools import find_packages
from setuptools import setup


version = "0.1.1"


if sys.argv[-1] == "release":
    # Release via github-actions.
    commands = [
        'git tag v{:s}'.format(version),
        'git push origin master --tag',
    ]
    for cmd in commands:
        print('+ {}'.format(cmd))
        subprocess.check_call(shlex.split(cmd))
    sys.exit(0)


setup_requires = []

with open('requirements.txt') as f:
    install_requires = []
    for line in f:
        req = line.split('#')[0].strip()
        install_requires.append(req)

setup(
    name="extendedos",
    version=version,
    description="Extended os module",
    author="iory",
    author_email="ab.ioryz@gmail.com",
    url="https://github.com/iory/eos",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    license="MIT",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
    packages=find_packages(),
    zip_safe=False,
    setup_requires=setup_requires,
    install_requires=install_requires,
    entry_points={
        "console_scripts": [
            "run-many=eos.apps.run_many:main"
        ]
    },
)
